@@ -250,6 +250,11 @@ class _Platform {
   static String _version() {
     throw new UnsupportedError("Platform._version");
   }
+
+  @patch
+  static String _localeName() {
+    throw new UnsupportedError("Platform._localeName");
+  }
 }
 
 @patch
@@ -285,6 +290,19 @@ class _ProcessUtils {
   }
 }
 
+@patch
+class ProcessInfo {
+  @patch
+  static int get currentRss {
+    throw new UnsupportedError("ProcessInfo.currentRss");
+  }
+
+  @patch
+  static int get maxRss {
+    throw new UnsupportedError("ProcessInfo.maxRss");
+  }
+}
+
 @patch
 class Process {
   @patch
@@ -422,6 +440,14 @@ class SecureSocket {
   }
 }
 
+@patch
+class RawSynchronousSocket {
+  @patch
+  static RawSynchronousSocket connectSync(host, int port) {
+    throw new UnsupportedError("RawSynchronousSocket.connectSync");
+  }
+}
+
 @patch
 class SecurityContext {
   @patch
