@@ -13,10 +13,8 @@ import 'package:analyzer/src/kernel/ast_from_analyzer.dart'
 
 import 'package:kernel/ast.dart' show Library, TreeNode;
 
-import 'package:front_end/src/fasta/builder/scope.dart' show Scope;
-
 import 'package:front_end/src/fasta/kernel/kernel_builder.dart'
-    show Builder, KernelFormalParameterBuilder;
+    show Builder, KernelFormalParameterBuilder, Scope;
 
 import 'element_store.dart' show ElementStore;
 
@@ -27,7 +25,7 @@ export 'element_store.dart' show ElementStore;
 TreeNode toKernel(
     AstNode node, ElementStore store, Library library, Scope scope) {
   ExpressionScope expressionScope = new ExpressionScope(store, library);
-  scope.local.forEach((String name, Builder builder) {
+  scope.forEach((String name, Builder builder) {
     if (builder is KernelFormalParameterBuilder) {
       LocalElement local = store[builder];
       assert(local != null);
