@@ -68,6 +68,16 @@ UnlinkedConstructorInitializer serializeConstructorInitializer(
  * serialization of a single constant [Expression].
  */
 abstract class AbstractConstExprSerializer {
+  /**
+   * Whether an expression that should be a constant is being serialized.
+   *
+   * For constants we need to store more than we need just for type inference,
+   * because we need to be able to restore these AST to evaluate actual values
+   * of constants. So, we need to store constructor arguments, elements for
+   * list and map literals even if these literals are typed.
+   */
+  final bool forConst;
+
   /**
    * See [UnlinkedExprBuilder.isValidConst].
    */
@@ -109,6 +119,8 @@ abstract class AbstractConstExprSerializer {
    */
   final List<EntityRefBuilder> references = <EntityRefBuilder>[];
 
+  AbstractConstExprSerializer(this.forConst);
+
   /**
    * Return `true` if the given [name] is a parameter reference.
    */
@@ -127,6 +139,12 @@ abstract class AbstractConstExprSerializer {
       _serialize(expr);
     } on StateError {
       isValidConst = false;
+      operations.clear();
+      assignmentOperators.clear();
+      ints.clear();
+      doubles.clear();
+      strings.clear();
+      references.clear();
     }
   }
 
@@ -269,12 +287,12 @@ abstract class AbstractConstExprSerializer {
 
   void _pushInt(int value) {
     assert(value >= 0);
-    if (value >= (1 << 32)) {
+    if (value >= 0x100000000) {
       int numOfComponents = 0;
       ints.add(numOfComponents);
       void pushComponents(int value) {
-        if (value >= (1 << 32)) {
-          pushComponents(value >> 32);
+        if (value >= 0x100000000) {
+          pushComponents(value ~/ 0x100000000);
         }
         numOfComponents++;
         ints.add(value & 0xFFFFFFFF);
@@ -363,7 +381,8 @@ abstract class AbstractConstExprSerializer {
     } else if (expr is AssignmentExpression) {
       _serializeAssignment(expr);
     } else if (expr is CascadeExpression) {
-      _serializeCascadeExpression(expr);
+      isValidConst = false;
+      _serialize(expr.target);
     } else if (expr is FunctionExpression) {
       isValidConst = false;
       List<int> indices = serializeFunctionExpression(expr);
@@ -388,6 +407,8 @@ abstract class AbstractConstExprSerializer {
       _serialize(expr.expression);
       references.add(serializeTypeName(expr.type));
       operations.add(UnlinkedExprOperation.typeCheck);
+    } else if (expr is SuperExpression) {
+      operations.add(UnlinkedExprOperation.pushSuper);
     } else if (expr is ThisExpression) {
       operations.add(UnlinkedExprOperation.pushThis);
     } else if (expr is ThrowExpression) {
@@ -404,21 +425,26 @@ abstract class AbstractConstExprSerializer {
   }
 
   void _serializeArguments(ArgumentList argumentList) {
-    List<Expression> arguments = argumentList.arguments;
-    // Serialize the arguments.
-    List<String> argumentNames = <String>[];
-    arguments.forEach((arg) {
-      if (arg is NamedExpression) {
-        argumentNames.add(arg.name.label.name);
-        _serialize(arg.expression);
-      } else {
-        _serialize(arg);
-      }
-    });
-    // Add numbers of named and positional arguments, and the op-code.
-    ints.add(argumentNames.length);
-    strings.addAll(argumentNames);
-    ints.add(arguments.length - argumentNames.length);
+    if (forConst) {
+      List<Expression> arguments = argumentList.arguments;
+      // Serialize the arguments.
+      List<String> argumentNames = <String>[];
+      arguments.forEach((arg) {
+        if (arg is NamedExpression) {
+          argumentNames.add(arg.name.label.name);
+          _serialize(arg.expression);
+        } else {
+          _serialize(arg);
+        }
+      });
+      // Add numbers of named and positional arguments, and the op-code.
+      ints.add(argumentNames.length);
+      strings.addAll(argumentNames);
+      ints.add(arguments.length - argumentNames.length);
+    } else {
+      ints.add(0);
+      ints.add(0);
+    }
   }
 
   void _serializeAssignment(AssignmentExpression expr) {
@@ -511,19 +537,14 @@ abstract class AbstractConstExprSerializer {
     }
   }
 
-  void _serializeCascadeExpression(CascadeExpression expr) {
-    _serialize(expr.target);
-    for (Expression section in expr.cascadeSections) {
-      operations.add(UnlinkedExprOperation.cascadeSectionBegin);
-      _serialize(section);
-      operations.add(UnlinkedExprOperation.cascadeSectionEnd);
-    }
-  }
-
   void _serializeListLiteral(ListLiteral expr) {
-    List<Expression> elements = expr.elements;
-    elements.forEach(_serialize);
-    ints.add(elements.length);
+    if (forConst || expr.typeArguments == null) {
+      List<Expression> elements = expr.elements;
+      elements.forEach(_serialize);
+      ints.add(elements.length);
+    } else {
+      ints.add(0);
+    }
     if (expr.typeArguments != null &&
         expr.typeArguments.arguments.length == 1) {
       references.add(serializeTypeName(expr.typeArguments.arguments[0]));
@@ -534,11 +555,15 @@ abstract class AbstractConstExprSerializer {
   }
 
   void _serializeMapLiteral(MapLiteral expr) {
-    for (MapLiteralEntry entry in expr.entries) {
-      _serialize(entry.key);
-      _serialize(entry.value);
+    if (forConst || expr.typeArguments == null) {
+      for (MapLiteralEntry entry in expr.entries) {
+        _serialize(entry.key);
+        _serialize(entry.value);
+      }
+      ints.add(expr.entries.length);
+    } else {
+      ints.add(0);
     }
-    ints.add(expr.entries.length);
     if (expr.typeArguments != null &&
         expr.typeArguments.arguments.length == 2) {
       references.add(serializeTypeName(expr.typeArguments.arguments[0]));
@@ -624,7 +649,9 @@ abstract class AbstractConstExprSerializer {
       references.add(ref);
       operations.add(UnlinkedExprOperation.pushReference);
     } else {
-      _serialize(expr.target);
+      if (!expr.isCascaded) {
+        _serialize(expr.target);
+      }
       strings.add(expr.propertyName.name);
       operations.add(UnlinkedExprOperation.extractProperty);
     }
