@@ -7,14 +7,13 @@ library analyzer.test.file_system.resource_uri_resolver_test;
 import 'package:analyzer/file_system/file_system.dart';
 import 'package:analyzer/file_system/memory_file_system.dart';
 import 'package:analyzer/src/generated/source.dart';
+import 'package:test/test.dart';
 import 'package:test_reflective_loader/test_reflective_loader.dart';
-import 'package:unittest/unittest.dart';
-
-import '../utils.dart';
 
 main() {
-  initializeTestEnvironment();
-  defineReflectiveTests(ResourceUriResolverTest);
+  defineReflectiveSuite(() {
+    defineReflectiveTests(ResourceUriResolverTest);
+  });
 }
 
 @reflectiveTest
@@ -25,8 +24,8 @@ class ResourceUriResolverTest {
   void setUp() {
     provider = new MemoryResourceProvider();
     resolver = new ResourceUriResolver(provider);
-    provider.newFile('/test.dart', '');
-    provider.newFolder('/folder');
+    provider.newFile(provider.convertPath('/test.dart'), '');
+    provider.newFolder(provider.convertPath('/folder'));
   }
 
   void test_creation() {
@@ -35,15 +34,15 @@ class ResourceUriResolverTest {
   }
 
   void test_resolveAbsolute_file() {
-    var uri = new Uri(scheme: 'file', path: '/test.dart');
+    var uri = provider.pathContext.toUri(provider.convertPath('/test.dart'));
     Source source = resolver.resolveAbsolute(uri);
     expect(source, isNotNull);
     expect(source.exists(), isTrue);
-    expect(source.fullName, '/test.dart');
+    expect(source.fullName, provider.convertPath('/test.dart'));
   }
 
   void test_resolveAbsolute_folder() {
-    var uri = new Uri(scheme: 'file', path: '/folder');
+    var uri = provider.pathContext.toUri(provider.convertPath('/folder'));
     Source source = resolver.resolveAbsolute(uri);
     expect(source, isNull);
   }
@@ -61,7 +60,7 @@ class ResourceUriResolverTest {
   }
 
   void test_restoreAbsolute() {
-    var uri = new Uri(scheme: 'file', path: '/test.dart');
+    var uri = provider.pathContext.toUri(provider.convertPath('/test.dart'));
     Source source = resolver.resolveAbsolute(uri);
     expect(source, isNotNull);
     expect(resolver.restoreAbsolute(source), uri);
