@@ -4,18 +4,26 @@
 
 library fasta.fasta_accessors;
 
-export 'package:kernel/frontend/accessors.dart' show wrapInvalid;
+export 'frontend_accessors.dart' show wrapInvalid;
 
-import 'package:kernel/frontend/accessors.dart'
-    show Accessor, buildIsNull, makeLet;
+import 'package:front_end/src/fasta/kernel/utils.dart' show offsetForToken;
+
+import 'package:front_end/src/scanner/token.dart' show Token;
+
+import 'frontend_accessors.dart' show Accessor, buildIsNull, makeLet;
+
+import 'package:front_end/src/fasta/builder/ast_factory.dart' show AstFactory;
+
+import 'package:front_end/src/fasta/type_inference/type_promotion.dart'
+    show TypePromoter;
 
 import 'package:kernel/ast.dart';
 
 import '../errors.dart' show internalError;
 
-import '../builder/scope.dart' show AccessErrorBuilder, ProblemBuilder;
+import '../scope.dart' show AccessErrorBuilder, ProblemBuilder, Scope;
 
-import 'package:kernel/frontend/accessors.dart' as kernel
+import 'frontend_accessors.dart' as kernel
     show
         IndexAccessor,
         NullAwarePropertyAccessor,
@@ -36,6 +44,12 @@ import '../names.dart' show callName;
 abstract class BuilderHelper {
   Uri get uri;
 
+  TypePromoter get typePromoter;
+
+  int get functionNestingLevel;
+
+  AstFactory get astFactory;
+
   Constructor lookupConstructor(Name name, {bool isSuper});
 
   Expression toSuperMethodInvocation(MethodInvocation node);
@@ -44,13 +58,22 @@ abstract class BuilderHelper {
 
   Member lookupSuperMember(Name name, {bool isSetter: false});
 
-  builderToFirstExpression(Builder builder, String name, int offset);
+  scopeLookup(Scope scope, String name, Token token,
+      {bool isQualified: false, PrefixBuilder prefix});
 
   finishSend(Object receiver, Arguments arguments, int offset);
 
   Expression buildCompileTimeError(error, [int offset]);
 
-  Initializer buildCompileTimeErrorIntializer(error, [int offset]);
+  Initializer buildInvalidIntializer(Expression expression, [int offset]);
+
+  Initializer buildSuperInitializer(
+      Constructor constructor, Arguments arguments,
+      [int offset]);
+
+  Initializer buildRedirectingInitializer(
+      Constructor constructor, Arguments arguments,
+      [int charOffset = -1]);
 
   Expression buildStaticInvocation(Procedure target, Arguments arguments);
 
@@ -59,6 +82,11 @@ abstract class BuilderHelper {
   Expression throwNoSuchMethodError(
       String name, Arguments arguments, int offset,
       {bool isSuper: false, isGetter: false, isSetter: false});
+
+  bool checkArguments(FunctionNode function, Arguments arguments,
+      List<TypeParameter> typeParameters);
+
+  StaticGet makeStaticGet(Member readTarget, Token token);
 }
 
 abstract class FastaAccessor implements Accessor {
@@ -70,13 +98,19 @@ abstract class FastaAccessor implements Accessor {
 
   String get plainNameForWrite => plainNameForRead;
 
+  bool get isInitializer => false;
+
   Expression buildForEffect() => buildSimpleRead();
 
   Initializer buildFieldInitializer(
       Map<String, FieldInitializer> initializers) {
-    // TODO(ahe): This error message is really bad.
-    return helper.buildCompileTimeErrorIntializer(
-        "Can't use $plainNameForRead here.", offset);
+    int offset = offsetForToken(token);
+    return helper.buildInvalidIntializer(
+        helper.buildCompileTimeError(
+            // TODO(ahe): This error message is really bad.
+            "Can't use $plainNameForRead here.",
+            offset),
+        offset);
   }
 
   Expression makeInvalidRead() {
@@ -84,21 +118,22 @@ abstract class FastaAccessor implements Accessor {
   }
 
   Expression makeInvalidWrite(Expression value) {
-    return buildThrowNoSuchMethodError(new Arguments(<Expression>[value]),
+    return buildThrowNoSuchMethodError(
+        helper.astFactory.arguments(<Expression>[value]),
         isSetter: true);
   }
 
-  /* Expression | FastaAccessor */ doInvocation(
+  /* Expression | FastaAccessor | Initializer */ doInvocation(
       int offset, Arguments arguments);
 
   /* Expression | FastaAccessor */ buildPropertyAccess(
       IncompleteSend send, bool isNullAware) {
     if (send is SendAccessor) {
-      return buildMethodInvocation(
-          buildSimpleRead(), send.name, send.arguments, send.offset,
+      return buildMethodInvocation(helper.astFactory, buildSimpleRead(),
+          send.name, send.arguments, offsetForToken(send.token),
           isNullAware: isNullAware);
     } else {
-      return PropertyAccessor.make(helper, send.offset, buildSimpleRead(),
+      return PropertyAccessor.make(helper, send.token, buildSimpleRead(),
           send.name, null, null, isNullAware);
     }
   }
@@ -106,19 +141,19 @@ abstract class FastaAccessor implements Accessor {
   /* Expression | FastaAccessor */ buildThrowNoSuchMethodError(
       Arguments arguments,
       {bool isSuper: false,
-      isGetter: false,
-      isSetter: false,
+      bool isGetter: false,
+      bool isSetter: false,
       String name,
       int offset}) {
-    return helper.throwNoSuchMethodError(
-        name ?? plainNameForWrite, arguments, offset ?? this.offset,
+    return helper.throwNoSuchMethodError(name ?? plainNameForWrite, arguments,
+        offset ?? offsetForToken(this.token),
         isGetter: isGetter, isSetter: isSetter, isSuper: isSuper);
   }
 
   bool get isThisPropertyAccessor => false;
 }
 
-abstract class CompileTimeErrorAccessor implements FastaAccessor {
+abstract class ErrorAccessor implements FastaAccessor {
   @override
   Expression get builtBinary => internalError("Unsupported operation.");
 
@@ -135,23 +170,35 @@ abstract class CompileTimeErrorAccessor implements FastaAccessor {
     internalError("Unsupported operation.");
   }
 
-  Expression buildError();
+  /// Pass [arguments] that must be evaluated before throwing an error.  At
+  /// most one of [isGetter] and [isSetter] should be true and they're passed
+  /// to [BuilderHelper.buildThrowNoSuchMethodError] if it is used.
+  Expression buildError(Arguments arguments,
+      {bool isGetter: false, bool isSetter: false, int offset});
 
   Name get name => internalError("Unsupported operation.");
 
+  @override
   String get plainNameForRead => name.name;
 
   withReceiver(Object receiver, {bool isNullAware}) => this;
 
+  @override
   Initializer buildFieldInitializer(
       Map<String, FieldInitializer> initializers) {
-    return new LocalInitializer(new VariableDeclaration.forValue(buildError()));
+    return helper.buildInvalidIntializer(
+        buildError(new Arguments.empty(), isSetter: true));
   }
 
-  doInvocation(int offset, Arguments arguments) => this;
+  @override
+  doInvocation(int offset, Arguments arguments) {
+    return buildError(arguments, offset: offset);
+  }
 
+  @override
   buildPropertyAccess(IncompleteSend send, bool isNullAware) => this;
 
+  @override
   buildThrowNoSuchMethodError(Arguments arguments,
       {bool isSuper: false,
       isGetter: false,
@@ -161,53 +208,73 @@ abstract class CompileTimeErrorAccessor implements FastaAccessor {
     return this;
   }
 
+  @override
   Expression buildAssignment(Expression value, {bool voidContext: false}) {
-    return buildError();
+    return buildError(helper.astFactory.arguments(<Expression>[value]),
+        isSetter: true);
   }
 
+  @override
   Expression buildCompoundAssignment(Name binaryOperator, Expression value,
       {int offset: TreeNode.noOffset,
       bool voidContext: false,
       Procedure interfaceTarget}) {
-    return buildError();
+    return buildError(helper.astFactory.arguments(<Expression>[value]),
+        isGetter: true);
   }
 
+  @override
   Expression buildPrefixIncrement(Name binaryOperator,
       {int offset: TreeNode.noOffset,
       bool voidContext: false,
       Procedure interfaceTarget}) {
-    return buildError();
+    return buildError(
+        helper.astFactory.arguments(<Expression>[new IntLiteral(1)]),
+        isGetter: true);
   }
 
+  @override
   Expression buildPostfixIncrement(Name binaryOperator,
       {int offset: TreeNode.noOffset,
       bool voidContext: false,
       Procedure interfaceTarget}) {
-    return buildError();
+    return buildError(
+        helper.astFactory.arguments(<Expression>[new IntLiteral(1)]),
+        isGetter: true);
   }
 
+  @override
   Expression buildNullAwareAssignment(Expression value, DartType type,
       {bool voidContext: false}) {
-    return buildError();
+    return buildError(helper.astFactory.arguments(<Expression>[value]),
+        isSetter: true);
   }
 
-  Expression buildSimpleRead() => buildError();
+  @override
+  Expression buildSimpleRead() =>
+      buildError(new Arguments.empty(), isGetter: true);
 
-  Expression makeInvalidRead() => buildError();
+  @override
+  Expression makeInvalidRead() =>
+      buildError(new Arguments.empty(), isGetter: true);
 
-  Expression makeInvalidWrite(Expression value) => buildError();
+  @override
+  Expression makeInvalidWrite(Expression value) {
+    return buildError(helper.astFactory.arguments(<Expression>[value]),
+        isSetter: true);
+  }
 }
 
 class ThisAccessor extends FastaAccessor {
   final BuilderHelper helper;
 
-  final int offset;
+  final Token token;
 
   final bool isInitializer;
 
   final bool isSuper;
 
-  ThisAccessor(this.helper, this.offset, this.isInitializer,
+  ThisAccessor(this.helper, this.token, this.isInitializer,
       {this.isSuper: false});
 
   @override
@@ -233,37 +300,44 @@ class ThisAccessor extends FastaAccessor {
       return new ThisExpression();
     } else {
       return helper.buildCompileTimeError(
-          "Can't use `super` as an expression.", offset);
+          "Can't use `super` as an expression.", offsetForToken(token));
     }
   }
 
   Initializer buildFieldInitializer(
       Map<String, FieldInitializer> initializers) {
     String keyword = isSuper ? "super" : "this";
-    return helper.buildCompileTimeErrorIntializer(
-        "Can't use '$keyword' here, did you mean '$keyword()'?", offset);
+    int offset = offsetForToken(token);
+    return helper.buildInvalidIntializer(
+        helper.buildCompileTimeError(
+            "Can't use '$keyword' here, did you mean '$keyword()'?", offset),
+        offset);
   }
 
   buildPropertyAccess(IncompleteSend send, bool isNullAware) {
     if (isInitializer && send is SendAccessor) {
       return buildConstructorInitializer(
-          send.offset, send.name, send.arguments);
+          offsetForToken(send.token), send.name, send.arguments);
     }
     if (send is SendAccessor) {
       // Notice that 'this' or 'super' can't be null. So we can ignore the
       // value of [isNullAware].
       MethodInvocation result = buildMethodInvocation(
-          new ThisExpression(), send.name, send.arguments, offset);
+          helper.astFactory,
+          new ThisExpression(),
+          send.name,
+          send.arguments,
+          offsetForToken(token));
       return isSuper ? helper.toSuperMethodInvocation(result) : result;
     } else {
       if (isSuper) {
         Member getter = helper.lookupSuperMember(send.name);
         Member setter = helper.lookupSuperMember(send.name, isSetter: true);
         return new SuperPropertyAccessor(
-            helper, send.offset, send.name, getter, setter);
+            helper, send.token, send.name, getter, setter);
       } else {
         return new ThisPropertyAccessor(
-            helper, send.offset, send.name, null, null);
+            helper, send.token, send.name, null, null);
       }
     }
   }
@@ -273,24 +347,25 @@ class ThisAccessor extends FastaAccessor {
       return buildConstructorInitializer(offset, new Name(""), arguments);
     } else {
       return buildMethodInvocation(
-          new ThisExpression(), callName, arguments, offset);
+          helper.astFactory, new ThisExpression(), callName, arguments, offset);
     }
   }
 
   Initializer buildConstructorInitializer(
       int offset, Name name, Arguments arguments) {
     Constructor constructor = helper.lookupConstructor(name, isSuper: isSuper);
-    Initializer result;
-    if (constructor == null) {
-      result = new LocalInitializer(new VariableDeclaration.forValue(
+    if (constructor == null ||
+        !helper.checkArguments(
+            constructor.function, arguments, <TypeParameter>[])) {
+      return helper.buildInvalidIntializer(
           buildThrowNoSuchMethodError(arguments,
-              isSuper: isSuper, name: name.name, offset: offset)));
+              isSuper: isSuper, name: name.name, offset: offset),
+          offset);
     } else if (isSuper) {
-      result = new SuperInitializer(constructor, arguments);
+      return helper.buildSuperInitializer(constructor, arguments, offset);
     } else {
-      result = new RedirectingInitializer(constructor, arguments);
+      return helper.buildRedirectingInitializer(constructor, arguments, offset);
     }
-    return result..fileOffset = offset;
   }
 
   Expression buildAssignment(Expression value, {bool voidContext: false}) {
@@ -326,21 +401,24 @@ class ThisAccessor extends FastaAccessor {
   Expression buildAssignmentError() {
     String message =
         isSuper ? "Can't assign to 'super'." : "Can't assign to 'this'.";
-    return helper.buildCompileTimeError(message, offset);
+    return helper.buildCompileTimeError(message, offsetForToken(token));
   }
 
-  toString() => "ThisAccessor($offset${isSuper ? ', super' : ''})";
+  toString() {
+    int offset = offsetForToken(token);
+    return "ThisAccessor($offset${isSuper ? ', super' : ''})";
+  }
 }
 
 abstract class IncompleteSend extends FastaAccessor {
   final BuilderHelper helper;
 
   @override
-  final int offset;
+  final Token token;
 
   final Name name;
 
-  IncompleteSend(this.helper, this.offset, this.name);
+  IncompleteSend(this.helper, this.token, this.name);
 
   @override
   Expression get builtBinary => internalError("Unsupported operation.");
@@ -361,22 +439,28 @@ abstract class IncompleteSend extends FastaAccessor {
   withReceiver(Object receiver, {bool isNullAware});
 }
 
-class IncompleteError extends IncompleteSend with CompileTimeErrorAccessor {
+class IncompleteError extends IncompleteSend with ErrorAccessor {
   final Object error;
 
-  IncompleteError(BuilderHelper helper, int offset, this.error)
-      : super(helper, offset, null);
+  IncompleteError(BuilderHelper helper, Token token, this.error)
+      : super(helper, token, null);
 
-  Expression buildError() {
-    return helper.buildCompileTimeError(error, offset);
+  @override
+  Expression buildError(Arguments arguments,
+      {bool isGetter: false, bool isSetter: false, int offset}) {
+    return helper.buildCompileTimeError(
+        error, offset ?? offsetForToken(this.token));
   }
+
+  @override
+  doInvocation(int offset, Arguments arguments) => this;
 }
 
 class SendAccessor extends IncompleteSend {
   final Arguments arguments;
 
-  SendAccessor(BuilderHelper helper, int offset, Name name, this.arguments)
-      : super(helper, offset, name) {
+  SendAccessor(BuilderHelper helper, Token token, Name name, this.arguments)
+      : super(helper, token, name) {
     assert(arguments != null);
   }
 
@@ -401,36 +485,42 @@ class SendAccessor extends IncompleteSend {
     }
     if (receiver is PrefixBuilder) {
       PrefixBuilder prefix = receiver;
-      receiver = helper.builderToFirstExpression(
-          prefix.exports[name.name], "${prefix.name}.${name.name}", offset);
-      return helper.finishSend(receiver, arguments, offset);
+      receiver = helper.scopeLookup(prefix.exports, name.name, token,
+          isQualified: true, prefix: prefix);
+      return helper.finishSend(receiver, arguments, offsetForToken(token));
     }
     Expression result;
     if (receiver is KernelClassBuilder) {
-      Builder builder = receiver.findStaticBuilder(name.name, offset, uri);
-      if (builder == null) {
+      Builder builder =
+          receiver.findStaticBuilder(name.name, offsetForToken(token), uri);
+      if (builder == null || builder is AccessErrorBuilder) {
         return buildThrowNoSuchMethodError(arguments);
       }
       if (builder.hasProblem) {
-        result = helper.buildProblemExpression(builder, offset);
+        result = helper.buildProblemExpression(builder, offsetForToken(token));
       } else {
         Member target = builder.target;
         if (target != null) {
           if (target is Field) {
-            result = buildMethodInvocation(new StaticGet(target), callName,
-                arguments, offset + (target.name?.name?.length ?? 0),
+            result = buildMethodInvocation(
+                helper.astFactory,
+                new StaticGet(target),
+                callName,
+                arguments,
+                offsetForToken(token) + (target.name?.name?.length ?? 0),
                 isNullAware: isNullAware);
           } else {
             result = helper.buildStaticInvocation(target, arguments)
-              ..fileOffset = offset;
+              ..fileOffset = offsetForToken(token);
           }
         } else {
-          result = buildThrowNoSuchMethodError(arguments)..fileOffset = offset;
+          result = buildThrowNoSuchMethodError(arguments)
+            ..fileOffset = offsetForToken(token);
         }
       }
     } else {
-      result = buildMethodInvocation(
-          helper.toValue(receiver), name, arguments, offset,
+      result = buildMethodInvocation(helper.astFactory,
+          helper.toValue(receiver), name, arguments, offsetForToken(token),
           isNullAware: isNullAware);
     }
     return result;
@@ -460,12 +550,15 @@ class SendAccessor extends IncompleteSend {
     return internalError("Unhandled");
   }
 
-  toString() => "SendAccessor($offset, $name, $arguments)";
+  toString() {
+    int offset = offsetForToken(token);
+    return "SendAccessor($offset, $name, $arguments)";
+  }
 }
 
 class IncompletePropertyAccessor extends IncompleteSend {
-  IncompletePropertyAccessor(BuilderHelper helper, int offset, Name name)
-      : super(helper, offset, name);
+  IncompletePropertyAccessor(BuilderHelper helper, Token token, Name name)
+      : super(helper, token, name);
 
   String get plainNameForRead => name.name;
 
@@ -487,11 +580,12 @@ class IncompletePropertyAccessor extends IncompleteSend {
     }
     if (receiver is PrefixBuilder) {
       PrefixBuilder prefix = receiver;
-      return helper.builderToFirstExpression(
-          prefix.exports[name.name], name.name, offset);
+      return helper.scopeLookup(prefix.exports, name.name, token,
+          isQualified: true, prefix: prefix);
     }
     if (receiver is KernelClassBuilder) {
-      Builder builder = receiver.findStaticBuilder(name.name, offset, uri);
+      Builder builder =
+          receiver.findStaticBuilder(name.name, offsetForToken(token), uri);
       if (builder == null) {
         // If we find a setter, [builder] is an [AccessErrorBuilder], not null.
         return buildThrowNoSuchMethodError(new Arguments.empty(),
@@ -501,15 +595,16 @@ class IncompletePropertyAccessor extends IncompleteSend {
       if (builder.isSetter) {
         setter = builder;
       } else if (builder.isGetter) {
-        setter =
-            receiver.findStaticBuilder(name.name, offset, uri, isSetter: true);
+        setter = receiver.findStaticBuilder(
+            name.name, offsetForToken(token), uri,
+            isSetter: true);
       } else if (builder.isField && !builder.isFinal) {
         setter = builder;
       }
-      return new StaticAccessor.fromBuilder(helper, builder, offset, setter);
+      return new StaticAccessor.fromBuilder(helper, builder, token, setter);
     }
-    return PropertyAccessor.make(helper, offset, helper.toValue(receiver), name,
-        null, null, isNullAware);
+    return PropertyAccessor.make(
+        helper, token, helper.toValue(receiver), name, null, null, isNullAware);
   }
 
   Expression buildNullAwareAssignment(Expression value, DartType type,
@@ -536,15 +631,18 @@ class IncompletePropertyAccessor extends IncompleteSend {
     return internalError("Unhandled");
   }
 
-  toString() => "IncompletePropertyAccessor($offset, $name)";
+  toString() {
+    int offset = offsetForToken(token);
+    return "IncompletePropertyAccessor($offset, $name)";
+  }
 }
 
 class IndexAccessor extends kernel.IndexAccessor with FastaAccessor {
   final BuilderHelper helper;
 
-  IndexAccessor.internal(this.helper, int offset, Expression receiver,
+  IndexAccessor.internal(this.helper, Token token, Expression receiver,
       Expression index, Procedure getter, Procedure setter)
-      : super.internal(receiver, index, getter, setter, offset);
+      : super.internal(helper, receiver, index, getter, setter, token);
 
   String get plainNameForRead => "[]";
 
@@ -552,23 +650,23 @@ class IndexAccessor extends kernel.IndexAccessor with FastaAccessor {
 
   Expression doInvocation(int offset, Arguments arguments) {
     return buildMethodInvocation(
-        buildSimpleRead(), callName, arguments, offset);
+        helper.astFactory, buildSimpleRead(), callName, arguments, offset);
   }
 
   toString() => "IndexAccessor()";
 
   static FastaAccessor make(
       BuilderHelper helper,
-      int offset,
+      Token token,
       Expression receiver,
       Expression index,
       Procedure getter,
       Procedure setter) {
     if (receiver is ThisExpression) {
-      return new ThisIndexAccessor(helper, offset, index, getter, setter);
+      return new ThisIndexAccessor(helper, token, index, getter, setter);
     } else {
       return new IndexAccessor.internal(
-          helper, offset, receiver, index, getter, setter);
+          helper, token, receiver, index, getter, setter);
     }
   }
 }
@@ -576,50 +674,50 @@ class IndexAccessor extends kernel.IndexAccessor with FastaAccessor {
 class PropertyAccessor extends kernel.PropertyAccessor with FastaAccessor {
   final BuilderHelper helper;
 
-  PropertyAccessor.internal(this.helper, int offset, Expression receiver,
+  PropertyAccessor.internal(this.helper, Token token, Expression receiver,
       Name name, Member getter, Member setter)
-      : super.internal(receiver, name, getter, setter, offset);
+      : super.internal(helper, receiver, name, getter, setter, token);
 
   String get plainNameForRead => name.name;
 
   bool get isThisPropertyAccessor => receiver is ThisExpression;
 
   Expression doInvocation(int offset, Arguments arguments) {
-    return buildMethodInvocation(receiver, name, arguments, offset);
+    return buildMethodInvocation(
+        helper.astFactory, receiver, name, arguments, offset);
   }
 
   toString() => "PropertyAccessor()";
 
   static FastaAccessor make(
       BuilderHelper helper,
-      int offset,
+      Token token,
       Expression receiver,
       Name name,
       Member getter,
       Member setter,
       bool isNullAware) {
     if (receiver is ThisExpression) {
-      return new ThisPropertyAccessor(helper, offset, name, getter, setter);
+      return new ThisPropertyAccessor(helper, token, name, getter, setter);
     } else {
       return isNullAware
           ? new NullAwarePropertyAccessor(
-              helper, offset, receiver, name, getter, setter, null)
+              helper, token, receiver, name, getter, setter, null)
           : new PropertyAccessor.internal(
-              helper, offset, receiver, name, getter, setter);
+              helper, token, receiver, name, getter, setter);
     }
   }
 }
 
 class StaticAccessor extends kernel.StaticAccessor with FastaAccessor {
-  final BuilderHelper helper;
-
-  StaticAccessor(this.helper, int offset, Member readTarget, Member writeTarget)
-      : super(readTarget, writeTarget, offset) {
+  StaticAccessor(
+      BuilderHelper helper, Token token, Member readTarget, Member writeTarget)
+      : super(helper, readTarget, writeTarget, token) {
     assert(readTarget != null || writeTarget != null);
   }
 
   factory StaticAccessor.fromBuilder(BuilderHelper helper, Builder builder,
-      int offset, Builder builderSetter) {
+      Token token, Builder builderSetter) {
     if (builder is AccessErrorBuilder) {
       AccessErrorBuilder error = builder;
       builder = error.builder;
@@ -636,15 +734,15 @@ class StaticAccessor extends kernel.StaticAccessor with FastaAccessor {
         setter = builderSetter.target;
       }
     }
-    return new StaticAccessor(helper, offset, getter, setter);
+    return new StaticAccessor(helper, token, getter, setter);
   }
 
   String get plainNameForRead => (readTarget ?? writeTarget).name.name;
 
   Expression doInvocation(int offset, Arguments arguments) {
     if (readTarget == null || isFieldOrGetter(readTarget)) {
-      return buildMethodInvocation(buildSimpleRead(), callName, arguments,
-          offset + (readTarget?.name?.name?.length ?? 0));
+      return buildMethodInvocation(helper.astFactory, buildSimpleRead(),
+          callName, arguments, offset + (readTarget?.name?.name?.length ?? 0));
     } else {
       return helper.buildStaticInvocation(readTarget, arguments)
         ..fileOffset = offset;
@@ -656,18 +754,16 @@ class StaticAccessor extends kernel.StaticAccessor with FastaAccessor {
 
 class SuperPropertyAccessor extends kernel.SuperPropertyAccessor
     with FastaAccessor {
-  final BuilderHelper helper;
-
-  SuperPropertyAccessor(
-      this.helper, int offset, Name name, Member getter, Member setter)
-      : super(name, getter, setter, offset);
+  SuperPropertyAccessor(BuilderHelper helper, Token token, Name name,
+      Member getter, Member setter)
+      : super(helper, name, getter, setter, token);
 
   String get plainNameForRead => name.name;
 
   Expression doInvocation(int offset, Arguments arguments) {
     if (getter == null || isFieldOrGetter(getter)) {
       return buildMethodInvocation(
-          buildSimpleRead(), callName, arguments, offset);
+          helper.astFactory, buildSimpleRead(), callName, arguments, offset);
     } else {
       return new DirectMethodInvocation(new ThisExpression(), getter, arguments)
         ..fileOffset = offset;
@@ -678,11 +774,9 @@ class SuperPropertyAccessor extends kernel.SuperPropertyAccessor
 }
 
 class ThisIndexAccessor extends kernel.ThisIndexAccessor with FastaAccessor {
-  final BuilderHelper helper;
-
-  ThisIndexAccessor(this.helper, int offset, Expression index, Procedure getter,
-      Procedure setter)
-      : super(index, getter, setter, offset);
+  ThisIndexAccessor(BuilderHelper helper, Token token, Expression index,
+      Procedure getter, Procedure setter)
+      : super(helper, index, getter, setter, token);
 
   String get plainNameForRead => "[]";
 
@@ -690,18 +784,16 @@ class ThisIndexAccessor extends kernel.ThisIndexAccessor with FastaAccessor {
 
   Expression doInvocation(int offset, Arguments arguments) {
     return buildMethodInvocation(
-        buildSimpleRead(), callName, arguments, offset);
+        helper.astFactory, buildSimpleRead(), callName, arguments, offset);
   }
 
   toString() => "ThisIndexAccessor()";
 }
 
 class SuperIndexAccessor extends kernel.SuperIndexAccessor with FastaAccessor {
-  final BuilderHelper helper;
-
-  SuperIndexAccessor(
-      this.helper, int offset, Expression index, Member getter, Member setter)
-      : super(index, getter, setter, offset);
+  SuperIndexAccessor(BuilderHelper helper, Token token, Expression index,
+      Member getter, Member setter)
+      : super(helper, index, getter, setter, token);
 
   String get plainNameForRead => "[]";
 
@@ -709,7 +801,7 @@ class SuperIndexAccessor extends kernel.SuperIndexAccessor with FastaAccessor {
 
   Expression doInvocation(int offset, Arguments arguments) {
     return buildMethodInvocation(
-        buildSimpleRead(), callName, arguments, offset);
+        helper.astFactory, buildSimpleRead(), callName, arguments, offset);
   }
 
   toString() => "SuperIndexAccessor()";
@@ -720,8 +812,8 @@ class ThisPropertyAccessor extends kernel.ThisPropertyAccessor
   final BuilderHelper helper;
 
   ThisPropertyAccessor(
-      this.helper, int offset, Name name, Member getter, Member setter)
-      : super(name, getter, setter, offset);
+      this.helper, Token token, Name name, Member getter, Member setter)
+      : super(helper, name, getter, setter, token);
 
   String get plainNameForRead => name.name;
 
@@ -734,7 +826,8 @@ class ThisPropertyAccessor extends kernel.ThisPropertyAccessor
       // `this.name.call(arguments)`.
       interfaceTarget = null;
     }
-    return buildMethodInvocation(new ThisExpression(), name, arguments, offset);
+    return buildMethodInvocation(
+        helper.astFactory, new ThisExpression(), name, arguments, offset);
   }
 
   toString() => "ThisPropertyAccessor()";
@@ -744,9 +837,9 @@ class NullAwarePropertyAccessor extends kernel.NullAwarePropertyAccessor
     with FastaAccessor {
   final BuilderHelper helper;
 
-  NullAwarePropertyAccessor(this.helper, int offset, Expression receiver,
+  NullAwarePropertyAccessor(this.helper, Token token, Expression receiver,
       Name name, Member getter, Member setter, DartType type)
-      : super(receiver, name, getter, setter, type, offset);
+      : super(helper, receiver, name, getter, setter, type, token);
 
   String get plainNameForRead => name.name;
 
@@ -758,47 +851,69 @@ class NullAwarePropertyAccessor extends kernel.NullAwarePropertyAccessor
 }
 
 class VariableAccessor extends kernel.VariableAccessor with FastaAccessor {
-  final BuilderHelper helper;
-
-  VariableAccessor(this.helper, int offset, VariableDeclaration variable,
+  VariableAccessor(
+      BuilderHelper helper, Token token, VariableDeclaration variable,
       [DartType promotedType])
-      : super(variable, promotedType, offset);
+      : super(helper, variable, promotedType, token);
 
   String get plainNameForRead => variable.name;
 
   Expression doInvocation(int offset, Arguments arguments) {
     // Normally the offset is at the start of the token, but in this case,
     // because we insert a '.call', we want it at the end instead.
-    return buildMethodInvocation(buildSimpleRead(), callName, arguments,
-        offset + (variable.name?.length ?? 0));
+    return buildMethodInvocation(helper.astFactory, buildSimpleRead(), callName,
+        arguments, offset + (variable.name?.length ?? 0));
   }
 
   toString() => "VariableAccessor()";
 }
 
 class ReadOnlyAccessor extends kernel.ReadOnlyAccessor with FastaAccessor {
-  final BuilderHelper helper;
-
   final String plainNameForRead;
 
-  ReadOnlyAccessor(
-      this.helper, Expression expression, this.plainNameForRead, int offset)
-      : super(expression, offset);
+  ReadOnlyAccessor(BuilderHelper helper, Expression expression,
+      this.plainNameForRead, Token token)
+      : super(helper, expression, token);
 
   Expression doInvocation(int offset, Arguments arguments) {
     return buildMethodInvocation(
-        buildSimpleRead(), callName, arguments, offset);
+        helper.astFactory, buildSimpleRead(), callName, arguments, offset);
   }
 }
 
 class ParenthesizedExpression extends ReadOnlyAccessor {
   ParenthesizedExpression(
-      BuilderHelper helper, Expression expression, int offset)
-      : super(helper, expression, "<a parenthesized expression>", offset);
+      BuilderHelper helper, Expression expression, Token token)
+      : super(helper, expression, "<a parenthesized expression>", token);
 
   Expression makeInvalidWrite(Expression value) {
     return helper.buildCompileTimeError(
-        "Can't assign to a parenthesized expression.", offset);
+        "Can't assign to a parenthesized expression.", offsetForToken(token));
+  }
+}
+
+class UnresolvedAccessor extends FastaAccessor with ErrorAccessor {
+  @override
+  final Token token;
+
+  @override
+  final BuilderHelper helper;
+
+  @override
+  final Name name;
+
+  UnresolvedAccessor(this.helper, this.name, this.token);
+
+  Expression doInvocation(int charOffset, Arguments arguments) {
+    return buildError(arguments, offset: charOffset);
+  }
+
+  @override
+  Expression buildError(Arguments arguments,
+      {bool isGetter: false, bool isSetter: false, int offset}) {
+    return helper.throwNoSuchMethodError(
+        plainNameForRead, arguments, offset ?? offsetForToken(this.token),
+        isGetter: isGetter, isSetter: isSetter);
   }
 }
 
@@ -806,20 +921,22 @@ bool isFieldOrGetter(Member member) {
   return member is Field || (member is Procedure && member.isGetter);
 }
 
-Expression buildMethodInvocation(
-    Expression receiver, Name name, Arguments arguments, int offset,
+Expression buildMethodInvocation(AstFactory astFactory, Expression receiver,
+    Name name, Arguments arguments, int offset,
     {bool isNullAware: false}) {
   if (isNullAware) {
     VariableDeclaration variable = new VariableDeclaration.forValue(receiver);
     return makeLet(
         variable,
         new ConditionalExpression(
-            buildIsNull(new VariableGet(variable)),
+            buildIsNull(astFactory, new VariableGet(variable)),
             new NullLiteral(),
-            new MethodInvocation(new VariableGet(variable), name, arguments)
+            astFactory.methodInvocation(
+                new VariableGet(variable), name, arguments)
               ..fileOffset = offset,
             const DynamicType()));
   } else {
-    return new MethodInvocation(receiver, name, arguments)..fileOffset = offset;
+    return astFactory.methodInvocation(receiver, name, arguments)
+      ..fileOffset = offset;
   }
 }
