@@ -8,6 +8,7 @@ library front_end.test.memory_file_system_test;
 import 'dart:convert';
 import 'dart:io' as io;
 
+import 'package:front_end/file_system.dart' show FileSystemException;
 import 'package:front_end/memory_file_system.dart';
 import 'package:path/path.dart' as pathos;
 import 'package:test/test.dart';
@@ -22,6 +23,9 @@ main() {
   });
 }
 
+const Matcher _throwsFileSystemException =
+    const Throws(const isInstanceOf<FileSystemException>());
+
 @reflectiveTest
 class FileTest extends _BaseTestNative {
   String path;
@@ -41,12 +45,38 @@ class FileTest extends _BaseTestNative {
     expect(file == entityForPath(join(tempPath, 'file.txt')), isTrue);
   }
 
+  test_exists_doesNotExist() async {
+    expect(await file.exists(), false);
+  }
+
+  test_exists_exists() async {
+    file.writeAsStringSync('x');
+    expect(await file.exists(), true);
+  }
+
   test_hashCode_samePath() {
     expect(file.hashCode, entityForPath(join(tempPath, 'file.txt')).hashCode);
   }
 
+  test_lastModified_doesNotExist() async {
+    expect(file.lastModified(), _throwsFileSystemException);
+  }
+
+  test_lastModified_increasesOnEachChange() async {
+    file.writeAsStringSync('x');
+    var mod1 = await file.lastModified();
+    file.writeAsStringSync('y');
+    var mod2 = await file.lastModified();
+    expect(mod2.isAfter(mod1), isTrue);
+
+    var file2 = entityForPath(join(tempPath, 'file2.txt'));
+    file2.writeAsStringSync('z');
+    var mod3 = await file2.lastModified();
+    expect(mod3.isAfter(mod2), isTrue);
+  }
+
   test_path() {
-    expect(file.uri, fileSystem.context.toUri(path));
+    expect(file.uri, context.toUri(path));
   }
 
   test_readAsBytes_badUtf8() async {
@@ -57,7 +87,7 @@ class FileTest extends _BaseTestNative {
   }
 
   test_readAsBytes_doesNotExist() {
-    expect(file.readAsBytes(), throwsException);
+    expect(file.readAsBytes(), _throwsFileSystemException);
   }
 
   test_readAsBytes_exists() async {
@@ -68,11 +98,11 @@ class FileTest extends _BaseTestNative {
 
   test_readAsString_badUtf8() {
     file.writeAsBytesSync([0xc0, 0x40]); // Invalid UTF-8
-    expect(file.readAsString(), throwsException);
+    expect(file.readAsString(), _throwsFileSystemException);
   }
 
   test_readAsString_doesNotExist() {
-    expect(file.readAsString(), throwsException);
+    expect(file.readAsString(), _throwsFileSystemException);
   }
 
   test_readAsString_exists() async {
@@ -122,7 +152,7 @@ abstract class MemoryFileSystemTestMixin extends _BaseTest {
 
   setUp() {
     super.setUp();
-    tempUri = fileSystem.context.toUri(tempPath);
+    tempUri = context.toUri(tempPath);
   }
 
   test_currentDirectory_trailingSlash() {
@@ -133,15 +163,11 @@ abstract class MemoryFileSystemTestMixin extends _BaseTest {
     var path = fileSystem.currentDirectory.path;
     var currentDirectoryWithoutSlash = fileSystem.currentDirectory
         .replace(path: path.substring(0, path.length - 1));
-    expect(
-        new MemoryFileSystem(fileSystem.context, currentDirectoryWithoutSlash)
-            .currentDirectory,
+    expect(new MemoryFileSystem(currentDirectoryWithoutSlash).currentDirectory,
         fileSystem.currentDirectory);
     // If the currentDirectory supplied to the MemoryFileSystem constructor
     // already has a trailing slash, no further trailing slash should be added.
-    expect(
-        new MemoryFileSystem(fileSystem.context, fileSystem.currentDirectory)
-            .currentDirectory,
+    expect(new MemoryFileSystem(fileSystem.currentDirectory).currentDirectory,
         fileSystem.currentDirectory);
   }
 
@@ -213,12 +239,13 @@ class MemoryFileSystemTestWindows extends _BaseTestWindows
     with MemoryFileSystemTestMixin {}
 
 abstract class _BaseTest {
+  pathos.Context get context;
   MemoryFileSystem get fileSystem;
 
   String get tempPath;
 
   MemoryFileSystemEntity entityForPath(String path) =>
-      fileSystem.entityForUri(fileSystem.context.toUri(path));
+      fileSystem.entityForUri(context.toUri(path));
 
   String join(String path1, String path2, [String path3, String path4]);
 
@@ -226,6 +253,7 @@ abstract class _BaseTest {
 }
 
 class _BaseTestNative extends _BaseTest {
+  final pathos.Context context = pathos.context;
   MemoryFileSystem fileSystem;
   String tempPath;
 
@@ -234,12 +262,12 @@ class _BaseTestNative extends _BaseTest {
 
   setUp() {
     tempPath = pathos.join(io.Directory.systemTemp.path, 'test_file_system');
-    fileSystem = new MemoryFileSystem(
-        pathos.context, pathos.toUri(io.Directory.current.path));
+    fileSystem = new MemoryFileSystem(pathos.toUri(io.Directory.current.path));
   }
 }
 
 class _BaseTestPosix extends _BaseTest {
+  final pathos.Context context = pathos.posix;
   MemoryFileSystem fileSystem;
   String tempPath;
 
@@ -248,11 +276,12 @@ class _BaseTestPosix extends _BaseTest {
 
   void setUp() {
     tempPath = '/test_file_system';
-    fileSystem = new MemoryFileSystem(pathos.posix, Uri.parse('file:///cwd'));
+    fileSystem = new MemoryFileSystem(Uri.parse('file:///cwd'));
   }
 }
 
 class _BaseTestWindows extends _BaseTest {
+  final pathos.Context context = pathos.windows;
   MemoryFileSystem fileSystem;
   String tempPath;
 
@@ -261,7 +290,6 @@ class _BaseTestWindows extends _BaseTest {
 
   void setUp() {
     tempPath = r'c:\test_file_system';
-    fileSystem =
-        new MemoryFileSystem(pathos.windows, Uri.parse('file:///c:/cwd'));
+    fileSystem = new MemoryFileSystem(Uri.parse('file:///c:/cwd'));
   }
 }
