@@ -42,8 +42,7 @@ class SystemEncoding extends Encoding {
   }
 }
 
-class _WindowsCodePageEncoder extends Converter<String, List<int>>
-    implements ChunkedConverter<String, List<int>, String, List<int>> {
+class _WindowsCodePageEncoder extends Converter<String, List<int>> {
   const _WindowsCodePageEncoder();
 
   List<int> convert(String input) {
@@ -93,8 +92,7 @@ class _WindowsCodePageEncoderSink extends StringConversionSinkBase {
   }
 }
 
-class _WindowsCodePageDecoder extends Converter<List<int>, String>
-    implements ChunkedConverter<List<int>, String, List<int>, String> {
+class _WindowsCodePageDecoder extends Converter<List<int>, String> {
   const _WindowsCodePageDecoder();
 
   String convert(List<int> input) {
